"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Sphere {
    constructor(cloudRequestor, cloudRequestorV2, id = null) {
        this.sphereId = null;
        this.rest = cloudRequestor;
        this.restV2 = cloudRequestorV2;
        this.sphereId = id;
    }
    async users() {
        return await this.rest.getUsers(this.sphereId);
    }
    async authorizationTokens() {
        return await this.rest.getSphereAuthorizationTokens(this.sphereId);
    }
    async crownstones() {
        return await this.rest.getCrownstonesInSphere(this.sphereId);
    }
    async locations() {
        return await this.rest.getLocationsInSphere(this.sphereId);
    }
    async keys() {
        let keys = await this.rest.getKeys();
        for (let i = 0; i < keys.length; i++) {
            if (keys[i].sphereId === this.sphereId) {
                return keys[i];
            }
        }
        return null;
    }
    async data() {
        return this.rest.getSphere(this.sphereId);
    }
    async presentPeople(ignoreDeviceId) {
        return this.rest.getPresentPeople(this.sphereId, ignoreDeviceId);
    }
    async setEnergyCollectionPermission(allowed) {
        return this.restV2.setEnergyUploadPermission(this.sphereId, allowed);
    }
    async getEnergyCollectionPermission() {
        return this.restV2.getEnergyUploadPermission(this.sphereId);
    }
    async uploadEnergyData(energyMeasurementData) {
        return this.restV2.postSphereEnergyUsage(this.sphereId, energyMeasurementData);
    }
    async getEnergyData(date, range) {
        date = new Date(date);
        let dates = getRange(date, range);
        return this.restV2.getSphereEnergyUsage(this.sphereId, dates.start.toISOString(), dates.end.toISOString(), range);
    }
}
exports.Sphere = Sphere;
function getRange(date, range) {
    if (range === "day") {
        let start = new Date(date.getFullYear(), date.getMonth(), date.getDate());
        let end = new Date(date.getFullYear(), date.getMonth(), date.getDate() + 1);
        return { start, end };
    }
    if (range === 'week') {
        // get the monday of the week of the date as start and a week later as end
        let start = new Date(date.getFullYear(), date.getMonth(), date.getDate() - (date.getDay() + 6) % 7);
        let end = new Date(start.getFullYear(), start.getMonth(), start.getDate() + 7);
        return { start, end };
    }
    if (range === 'month') {
        let start = new Date(date.getFullYear(), date.getMonth(), 1);
        let end = new Date(date.getFullYear(), date.getMonth() + 1, 1);
        return { start, end };
    }
    if (range === 'year') {
        let start = new Date(date.getFullYear(), 0, 1);
        let end = new Date(date.getFullYear() + 1, 0, 1);
        return { start, end };
    }
}
//# sourceMappingURL=sphere.js.map