"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Util_1 = require("../util/Util");
const requestorBase_1 = require("./requestorBase");
const crownstoneRequests_1 = require("./requests/crownstoneRequests");
const sphereRequests_1 = require("./requests/sphereRequests");
const userRequests_1 = require("./requests/userRequests");
const hubRequests_1 = require("./requests/hubRequests");
const locationRequests_1 = require("./requests/locationRequests");
const webhookRequests_1 = require("./requests/webhookRequests");
const sphereV2Requests_1 = require("./requests/nextCloud/sphereV2Requests");
const crownstoneV2Requests_1 = require("./requests/nextCloud/crownstoneV2Requests");
class CloudRequestor extends requestorBase_1.RequestorBase {
    constructor(tokenStore, cache, customEndpoint = 'https://cloud.ownstone.org/api/') {
        super(tokenStore, cache);
        this.setEndpoint(customEndpoint);
    }
    isUser() {
        return this.tokenStore.cloudUser.userId !== undefined;
    }
    isHub() {
        return this.tokenStore.cloudHub.hubToken !== undefined;
    }
    interface() {
        // @ts-ignore
        return this;
    }
}
exports.CloudRequestor = CloudRequestor;
class CloudRequestorV2 extends requestorBase_1.RequestorBase {
    constructor(tokenStore, cache, customEndpoint = 'https://next.ownstone.org/api/') {
        super(tokenStore, cache);
        this.setEndpoint(customEndpoint);
    }
    isUser() {
        return this.tokenStore.cloudUser.userId !== undefined;
    }
    isHub() {
        return this.tokenStore.cloudHub.hubToken !== undefined;
    }
    interface() {
        // @ts-ignore
        return this;
    }
}
exports.CloudRequestorV2 = CloudRequestorV2;
Util_1.Util.applyMixins(CloudRequestor, [
    userRequests_1.UserRequests,
    locationRequests_1.LocationRequests,
    hubRequests_1.HubRequests,
    sphereRequests_1.SphereRequests,
    crownstoneRequests_1.CrownstoneRequests
]);
Util_1.Util.applyMixins(CloudRequestorV2, [
    sphereV2Requests_1.SphereV2Requests,
    crownstoneV2Requests_1.CrownstoneV2Requests
]);
class WebhookRequestor extends requestorBase_1.RequestorBase {
    constructor(tokenStore, cache, customEndpoint = 'https://webhooks.ownstone.org/api/') {
        super(tokenStore, cache);
        this.setEndpoint(customEndpoint);
    }
    interface() {
        // @ts-ignore
        return this;
    }
}
exports.WebhookRequestor = WebhookRequestor;
Util_1.Util.applyMixins(WebhookRequestor, [
    webhookRequests_1.WebhookRequests,
]);
//# sourceMappingURL=requestors.js.map