"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const got_1 = __importDefault(require("got"));
const Logger_1 = require("../Logger");
const log = Logger_1.Logger(__filename);
async function req(type, url, options, silent = false) {
    var _a, _b, _c, _d;
    let token = Math.floor(Math.random() * 1e8).toString(36);
    if (silent === false) {
        logRequest(type, url, options, token);
    }
    if (options.timeout === undefined) {
        options.timeout = 30000;
    }
    let result;
    try {
        switch (type) {
            case "POST":
                result = await got_1.default.post(url, options);
                break;
            case "GET":
                result = await got_1.default.get(url, options);
                break;
            case "DELETE":
                result = await got_1.default.delete(url, options);
                break;
            case "PUT":
                result = await got_1.default.put(url, options);
                break;
            case "PATCH":
                result = await got_1.default.patch(url, options);
                break;
        }
        if (silent === false) {
            log.debug("Request result:", result.statusCode, result.body, token);
            log.info("Request successful:", result.statusCode, token);
        }
        return result;
    }
    catch (err) {
        if (err.response) {
            // response error
            let statusCode = err.response.statusCode;
            let body = err.response.body;
            let messageInBody = (_b = (_a = err.response.body) === null || _a === void 0 ? void 0 : _a.err) === null || _b === void 0 ? void 0 : _b.message;
            let codeInBody = (_d = (_c = err.response.body) === null || _c === void 0 ? void 0 : _c.err) === null || _d === void 0 ? void 0 : _d.code;
            let error = { statusCode, body };
            if (messageInBody) {
                err.message = messageInBody;
            }
            if (codeInBody) {
                err.code = codeInBody;
            }
            if (silent === false) {
                log.error("Something went wrong with request", token, error);
            }
            throw error;
        }
        else if (err.request) {
            // error during request
        }
        if (silent === false) {
            log.error("Something went wrong with request", token, err);
        }
        throw err;
    }
}
exports.req = req;
function logRequest(type, url, options, token) {
    let headers = options.headers || null;
    let queryParams = options.searchParams || null;
    let bodyParams = options.json || null;
    let responseType = options.responseType;
    log.info("sending", type, "request to", url, 'headers:', headers, 'queryParams:', queryParams, 'bodyParams:', bodyParams, 'responseType', responseType, token);
}
//# sourceMappingURL=request.js.map